require_relative "../lib/geo_seeder"

describe "GeoSeeder" do
  context "Location" do
    
    describe "api_request" do
      it "returns an array of result google address hashes for valid request input" do
        response = GeoSeeder::Location.api_request({latlng: [40.7492998, -73.8888135]})
        expect(response.is_a?(Array)).to be true
        expect(response.first.is_a?(Hash)).to be true
        expect(response.first["formatted_address"]).to match(/NY 11372/)
      end
      
      it "returns an empty array for requests that yield no results" do
        response = GeoSeeder::Location.api_request({address: "asdfjkhsdf"})
        expect(response.is_a?(Array)).to be true
        expect(response).to be_empty
      end
    end
    
    describe "latlng" do
      let(:location) { GeoSeeder::Location.latlng("77-11 37th Ave, Jackson Heights, NY 11372") }
      
      it "returns an a Location object" do
        expect(location.class).to eq(GeoSeeder::Location)
      end
      
      it "returns an object with lat/lng attributes set as floats" do
        expect(location.lat.class).to eq(Float)
        expect(location.lng.class).to eq(Float)
      end
      
      it "represent the closest approximation of the provided address (if valid)" do
        expect(location.formatted_address).to match(/77-11 37th Ave/)
      end
      
      it "retruns nil for invalid input" do
        bad_location = GeoSeeder::Location.latlng("asdfasdf")
        expect(bad_location).to be_nil
      end
    end
    
    describe "address" do
      let(:location) { GeoSeeder::Location.address([40.7307596, -73.9913105]) }
      
      it "returns an a Location object" do
        expect(location.class).to eq(GeoSeeder::Location)
      end
      
      it "returns an object with proper address attributes set" do
        expect(location.street).not_to be_nil
        expect(location.city).not_to be_nil
        expect(location.country).not_to be_nil
        expect(location.state).not_to be_nil
        expect(location.zip_code).not_to be_nil
      end
      
      it "retruns nil for invalid input" do
        bad_location = GeoSeeder::Location.latlng([3441283, 88942838])
        expect(bad_location).to be_nil
      end
    end
    
    describe "all" do
      let(:loc_attrs) { GeoSeeder::Location.latlng("Brooklyn, NY").all }
      
      it "returns a hash of all attributes of a location object" do
        expect(loc_attrs.is_a?(Hash)).to be true
      end
      
      it "only returns hash with keys of instance variables possessed by Location object" do
        expect(loc_attrs[:city]).to eq("Brooklyn")
        expect(loc_attrs.has_key?(:street_number)).to be false
      end
    end
    
    describe "random" do
      let(:locations) {
        GeoSeeder::Location.random({
          center: "54 LeFever Ln, New Paltz, NY 12561",
          radius: 5,
          quantity: 3
        })
      }
      
      it "retruns an array of Location objects of length - quantity" do
        expect(locations.is_a?(Array)).to be true
        expect(locations.sample.class).to eq(GeoSeeder::Location)
        expect(locations.count).to eq(3)
      end
      
      it "only returns objects with full addresses, i.e. only addresses with street_numbers" do
        expect(locations.all? { |loc| loc.street_number =~ /\d+/}).to be true
      end
    end
  end
end